@@ -12,16 +12,20 @@ from oasys.widgets import gui as oasysgui
 from oasys.widgets import congruence
 from oasys.util.oasys_util import TriggerIn, TriggerOut, EmittingStream
 
-from syned.beamline.beamline import Beamline
 from syned.storage_ring.magnetic_structures.undulator import Undulator
 
 from wofry.propagator.wavefront1D.generic_wavefront import GenericWavefront1D
 from wofry.propagator.util.gaussian_schell_model import GaussianSchellModel1D
 
 from orangecontrib.wofry.util.wofry_objects import WofryData
-from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget
-from orangecontrib.xoppy.util.python_script import PythonScript  # TODO: change import from wofry!!!
+from orangecontrib.esrf.wofry.widgets.gui.ow_wofry_widget import WofryWidget # TODO: from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget
 
+from orangecontrib.esrf.wofry.util.wofry_light_source import WOLightSource
+from syned.beamline.beamline import Beamline
+from orangecontrib.esrf.wofry.util.wofry_beamline import WOBeamline # TODO: from wofry...
+
+
+import scipy.constants as codata
 
 class OWUndulatorGaussianShellModel1D(WofryWidget):
 
@@ -71,17 +75,7 @@ class OWUndulatorGaussianShellModel1D(WofryWidget):
 
     def __init__(self):
 
-        super().__init__(is_automatic=False, show_view_options=True)
-
-        #
-        # add script tab to tabs panel
-        #
-        script_tab = oasysgui.createTabPage(self.main_tabs, "Script")
-        self.wofry_script = PythonScript()
-        self.wofry_script.code_area.setFixedHeight(400)
-        script_box = gui.widgetBox(script_tab, "Python script", addSpace=True, orientation="horizontal")
-        script_box.layout().addWidget(self.wofry_script)
-
+        super().__init__(is_automatic=False, show_view_options=True, show_script_tab=True)
 
         self.runaction = widget.OWAction("Generate Wavefront", self)
         self.runaction.triggered.connect(self.generate)
@@ -312,8 +306,117 @@ class OWUndulatorGaussianShellModel1D(WofryWidget):
 
                 self.generate()
 
+    def get_light_source(self, sigmaI, beta):
+
+        return WOLightSource(
+            name                = self.name                ,
+            # electron_beam       = None  ,
+            # magnetic_structure  = None  ,
+            dimension           = 1           ,
+            initialize_from     = self.initialize_from     ,
+            range_from_h        = self.range_from        ,
+            range_to_h          = self.range_to          ,
+            # range_from_v        = None       ,
+            # range_to_v          = None       ,
+            steps_start_h       = self.steps_start       ,
+            steps_step_h        = self.steps_step        ,
+            steps_start_v       = None       ,
+            # steps_step_v        = None       ,
+            number_of_points_h  = self.number_of_points  ,
+            # number_of_points_v  = None  ,
+            energy              = self.photon_energy       ,
+            sigma_h             = sigmaI             ,
+            # sigma_v             = None             ,
+            amplitude           = 1.0           ,
+            kind_of_wave        = (3 if (self.use_emittances > 0) else 2)  ,
+            n_h                 = self.mode_index                 ,
+            # n_v                 = None                 ,
+            beta_h              = beta              ,
+            # beta_v              = None          ,
+            units               = 0,
+            # wavelength          = 0,
+            # initialize_amplitude= 0,
+            # complex_amplitude_re= 0,
+            # complex_amplitude_im= 0,
+            # phase               = 0,
+            # radius              = 0,
+            # center              = 0,
+            # inclination         = 0,
+            # gaussian_shift      = 0,
+            # add_random_phase    = 0,
+        )
+
+    def calculate_gsm_parameters(self):
+        #
+        # calculations
+        #
+        wavelength = codata.h * codata.c / codata.e / self.photon_energy
+
+        sigma_r = 2.740 / 4 / numpy.pi * numpy.sqrt(wavelength * self.undulator_length)
+        sigma_r_prime = 0.69 * numpy.sqrt(wavelength / self.undulator_length)
+
+        print("Radiation values at photon energy=%f eV:" % self.photon_energy)
+        print("   intensity sigma      : %6.3f um,  FWHM: %6.3f um" % (sigma_r * 1e6, sigma_r * 2.355e6))
+        print("   intensity sigmaprime: %6.3f urad, FWHM: %6.3f urad" % (sigma_r_prime * 1e6, sigma_r_prime * 2.355e6))
+
+        q = 0
+        number_of_modes = 0
+        if self.use_emittances == 0:
+            sigmaI = sigma_r
+            beta = None
+
+        else:
+            Sx = numpy.sqrt(sigma_r ** 2 + self.sigma_h ** 2)
+            Sxp = numpy.sqrt(sigma_r_prime ** 2 + self.sigma_divergence_h ** 2)
+            Sy = numpy.sqrt(sigma_r ** 2 + self.sigma_v ** 2)
+            Syp = numpy.sqrt(sigma_r_prime ** 2 + self.sigma_divergence_v ** 2)
+            print("\nElectron beam values:")
+            print("   sigma_h : %6.3f um, sigma_v: %6.3f um\n" % (self.sigma_h * 1e6, self.sigma_v * 1e6))
+            print("\nPhoton beam values (convolution):")
+            print("   SIGMA_H p: %6.3f um, SIGMA_V: %6.3f um\n" % (Sx * 1e6, Sy * 1e6))
+            print("   SIGMA_H' : %6.3f urad, SIGMA_V': %6.3f urad\n" % (Sxp * 1e6, Syp * 1e6))
+
+            labels = ["", "H", "V"]
+            if self.use_emittances == 1:
+                cf = sigma_r * sigma_r_prime / Sx / Sxp
+                sigmaI = Sx
+            elif self.use_emittances == 2:
+                cf = sigma_r * sigma_r_prime / Sy / Syp
+                sigmaI = Sy
+
+            print("\nCoherence fraction (from %s emittance): %f" % (labels[self.use_emittances], cf))
+
+            sigmaI *= self.correction_factor
+            beta = cf / numpy.sqrt(1 - cf)
+            sigmaMu = beta * sigmaI
+
+            print("\nGaussian Shell-model (matching coherence fraction in %s direction):" % \
+                  labels[self.use_emittances])
+            print("       beta: %6.3f" % beta)
+            print("       sigmaI : %6.3f um" % (sigmaI * 1e6))
+            print("       sigmaMu: %6.3f um" % (sigmaMu * 1e6))
+
+            q = 1.0 / (1 + beta ** 2 / 2 + beta * numpy.sqrt(1 + (beta / 2) ** 2))
+
+            # print(">>>>", cumulated_occupation, 1.0 - cumulated_occupation)
+            number_of_modes = int(numpy.log(1.0 - self.spectral_density_threshold) / numpy.log(q))
+
+            if number_of_modes < 1: number_of_modes = 1
+
+            print("\nTo consider %f of spectral density in %s we need %d modes." % \
+                  (self.spectral_density_threshold, labels[self.use_emittances], number_of_modes))
+
+            # _gsm_1d = GaussianSchellModel1D(1.0, sigmaI, sigmaMu)
+
+            # self.wavefront1D.set_gaussian_hermite_mode(
+            #     sigmaI, self.mode_index, amplitude=1.0, shift=0.0, beta=beta)
+
+        return sigmaI, beta, number_of_modes, q
+
     def generate(self):
-        try:
+
+        method = 1
+        if method == 0:
             self.wofry_output.setText("")
 
             sys.stdout = EmittingStream(textWritten=self.writeStdOut)
@@ -329,66 +432,12 @@ class OWUndulatorGaussianShellModel1D(WofryWidget):
                 self.wavefront1D = GenericWavefront1D.initialize_wavefront_from_steps(x_start=self.steps_start, x_step=self.steps_step, number_of_points=self.number_of_points)
 
             self.wavefront1D.set_photon_energy(self.photon_energy)
-            wavelength = self.wavefront1D.get_wavelength()
 
-
-            sigma_r = 2.740 / 4 / numpy.pi * numpy.sqrt(wavelength * self.undulator_length)
-            sigma_r_prime = 0.69 * numpy.sqrt(wavelength / self.undulator_length)
-
-
-            print("Radiation values at photon energy=%f eV:" % self.photon_energy)
-            print("   intensity sigma      : %6.3f um,  FWHM: %6.3f um" % (sigma_r * 1e6, sigma_r * 2.355e6))
-            print("   intensity sigmaprime: %6.3f urad, FWHM: %6.3f urad" % (sigma_r_prime * 1e6, sigma_r_prime * 2.355e6))
+            sigmaI, beta, _n, q = self.calculate_gsm_parameters()
 
             if self.use_emittances == 0:
-                sigmaI = sigma_r
                 self.wavefront1D.set_gaussian(sigmaI, amplitude=1.0)
-                beta = None
-
             else:
-                Sx  = numpy.sqrt( sigma_r**2       + self.sigma_h**2)
-                Sxp = numpy.sqrt( sigma_r_prime**2 + self.sigma_divergence_h**2)
-                Sy  = numpy.sqrt( sigma_r**2       + self.sigma_v**2)
-                Syp = numpy.sqrt( sigma_r_prime**2 + self.sigma_divergence_v**2)
-                print("\nElectron beam values:")
-                print("   sigma_h : %6.3f um, sigma_v: %6.3f um\n" % (self.sigma_h * 1e6, self.sigma_v * 1e6))
-                print("\nPhoton beam values (convolution):")
-                print("   SIGMA_H p: %6.3f um, SIGMA_V: %6.3f um\n" % (Sx * 1e6, Sy * 1e6))
-                print("   SIGMA_H' : %6.3f urad, SIGMA_V': %6.3f urad\n" % (Sxp * 1e6, Syp * 1e6))
-
-                labels = ["", "H", "V"]
-                if self.use_emittances == 1:
-                    cf = sigma_r * sigma_r_prime / Sx / Sxp
-                    sigmaI = Sx
-                elif self.use_emittances == 2:
-                    cf = sigma_r * sigma_r_prime / Sy / Syp
-                    sigmaI = Sy
-
-                print("\nCoherence fraction (from %s emittance): %f" % (labels[self.use_emittances], cf))
-
-                sigmaI *= self.correction_factor
-                beta = cf / numpy.sqrt(1 - cf)
-                sigmaMu = beta * sigmaI
-
-                print("\nGaussian Shell-model (matching coherence fraction in %s direction):" % \
-                      labels[self.use_emittances])
-                print("       beta: %6.3f" % beta)
-                print("       sigmaI : %6.3f um" % (sigmaI  * 1e6))
-                print("       sigmaMu: %6.3f um" % (sigmaMu * 1e6))
-
-                q = 1.0 / (1 + beta ** 2 / 2 + beta * numpy.sqrt(1 + (beta / 2) ** 2))
-
-                # print(">>>>", cumulated_occupation, 1.0 - cumulated_occupation)
-                _n = int(numpy.log(1.0 - self.spectral_density_threshold) / numpy.log(q))
-
-
-                if _n < 1: _n = 1
-
-                print("\nTo consider %f of spectral density in %s we need %d modes." % \
-                      (self.spectral_density_threshold, labels[self.use_emittances], _n))
-
-                _gsm_1d = GaussianSchellModel1D(1.0, sigmaI, sigmaMu)
-
                 self.wavefront1D.set_gaussian_hermite_mode(
                     sigmaI, self.mode_index, amplitude=1.0, shift=0.0, beta=beta)
 
@@ -409,12 +458,48 @@ class OWUndulatorGaussianShellModel1D(WofryWidget):
 
             self.send("WofryData", WofryData(wavefront=self.wavefront1D))
 
-        except Exception as exception:
-            QMessageBox.critical(self, "Error", str(exception), QMessageBox.Ok)
+        else:
+            self.wofry_output.setText("")
 
-            if self.IS_DEVELOP: raise exception
+            sys.stdout = EmittingStream(textWritten=self.writeStdOut)
+
+
+            self.progressBarInit()
+
+            self.check_fields()
+
+
+            sigmaI, beta, _n, q = self.calculate_gsm_parameters()
+
+            light_source = self.get_light_source(sigmaI, beta)
+            self.wavefront1D = light_source.get_wavefront()
+
+
+            if self.use_emittances == 0:
+                self._cumulated_occupation = numpy.array([1.0])
+            else:
+                indices = numpy.arange(_n)
+                self._cumulated_occupation = (1.0 - q ** (indices+1))
+
+            if self.view_type != 0:
+                self.initializeTabs()
+                self.plot_results()
+            else:
+                self.progressBarFinished()
+
+
+            # try:
+            #     self.wofry_script.set_code(self.generate_python_code(sigmaI,beta))
+            # except:
+            #     pass
+
+            beamline = WOBeamline(light_source=light_source)
+            try:
+                self.wofry_script.set_code(beamline.to_python_code())
+            except:
+                pass
+            self.send("WofryData", WofryData(wavefront=self.wavefront1D, beamline=beamline))
 
-            self.progressBarFinished()
 
     def generate_python_code(self,sigmaI,beta=1.0):
 
@@ -451,7 +536,6 @@ class OWUndulatorGaussianShellModel1D(WofryWidget):
 
             self.plot_data1D(1e6 * self.wavefront1D.get_abscissas(),
                              self.wavefront1D.get_intensity(),
-                             # self._cumulated_occupation / self._cumulated_occupation[-1],
                              progressBarValue=progressBarValue,
                              tabs_canvas_index=0,
                              plot_canvas_index=0,
@@ -462,7 +546,6 @@ class OWUndulatorGaussianShellModel1D(WofryWidget):
 
 
             self.plot_data1D(numpy.arange(self._cumulated_occupation.size),
-                             # self.wavefront1D.get_intensity(),
                              self._cumulated_occupation,
                              progressBarValue=progressBarValue,
                              tabs_canvas_index=1,
@@ -473,34 +556,6 @@ class OWUndulatorGaussianShellModel1D(WofryWidget):
                              calculate_fwhm=False)
 
 
-            #
-            #
-            # nx, ny = self._eigenvalues_map.shape
-            #
-            # tx = 1
-            # ty = 1
-            # if nx > 1:
-            #     dataX = numpy.arange(nx) * nx / (nx - 1)
-            # else:
-            #     dataX = numpy.array([0,2])
-            #
-            # if ny > 1:
-            #     dataY = numpy.arange(ny) * ny / (ny - 1)
-            # else:
-            #     dataY = numpy.array([0,2])
-            #
-            #
-            # self.plot_data2D(data2D=self._eigenvalues_map / self._eigenvalues_map[0,0],
-            #                  dataX=dataX,
-            #                  dataY=dataY,
-            #                  progressBarValue=progressBarValue,
-            #                  tabs_canvas_index=2,
-            #                  plot_canvas_index=2,
-            #                  title="Eigenvalue(n,m) / Eigenvalue(0,0)",
-            #                  xtitle="H mode index n",
-            #                  ytitle="V mode index m")
-
-
             self.progressBarFinished()
 
 
